#include <GuiEdit.au3>
#include <GuiListView.au3>
#include <GUIConstantsEx.au3>
#include <StaticConstants.au3>
#include <WindowsConstants.au3>
#include <WinAPIvkeysConstants.au3>
#include <WinAPIShellEx.au3>
#include "Functions.au3"

Opt( "MustDeclareVars", 1 )

Global $iGuiWidth = 600, $iGuiHeight = 400
Global $hListView, $iLVCols = 100, $iItem = -1, $iSubItem = 0, $iSubItemPrev = -1, $aRect, $hHeader, $bLVHasFocus = False, $idSetLeftMostVisibleCellFocused
Global $hEdit, $idEditOpen, $idEditClose, $bEditOpen = False, $bDoNotOpenControl = False, $bEditEscape = False
Global $bControl = False

; You can define a few settings with the globals below:

; Open Edit control on Enter key
Global $bEditOpenOnEnter = True

; Open Edit control on Space key
Global $bEditOpenOnSpace = False

; Open Edit control on double or single click
Global $bEditOpenOnDoubleClick = True   ; Open Edit control on double click in ListView cell
;Global $bEditOpenOnDoubleClick = False ; Open Edit control on single click in ListView cell

Example()

Func Example()
	; Create GUI
	Local $hGui = GUICreate( "Edit ListView cells with Edit control", $iGuiWidth, $iGuiHeight, -1, -1, $WS_OVERLAPPEDWINDOW )

	; Create ListView
	Local $idListView = GUICtrlCreateListView( "", 10, 10, $iGuiWidth-20, $iGuiHeight-20, $GUI_SS_DEFAULT_LISTVIEW-$LVS_SINGLESEL, $WS_EX_CLIENTEDGE )
	_GUICtrlListView_SetExtendedListViewStyle( $idListView, $LVS_EX_DOUBLEBUFFER+$LVS_EX_FULLROWSELECT+$LVS_EX_HEADERDRAGDROP )
	$hHeader = _GUICtrlListView_GetHeader( $idListView )
	$hListView = GUICtrlGetHandle( $idListView )

	; Add columns
	For $i = 1 To $iLVCols
		_GUICtrlListView_AddColumn( $hListView, "Column " & $i, 94 )
	Next
	$aColumnOrder = _GUICtrlListView_GetColumnOrderArray( $hListView )
	$aColumnOrderReverse = $aColumnOrder
	For $i = 1 To $aColumnOrder[0]
		$aColumnOrderReverse[$aColumnOrder[$i]] = $i
	Next

	; Add rows
	; This can lead to resized columns to get the text to fit
	Local $sRow
	For $i = 0 To 199
		$sRow = "Row " & $i+1 & " / Col 1"
		For $j = 2 To $iLVCols
			$sRow &= "|Row " & $i+1 & " / Col " & $j
		Next
		GUICtrlCreateListViewItem( $sRow, $idListView )
	Next

	; Get column widths after a possible change of widths
	Dim $aColumnWidths[$iLVCols]
	Dim $aColLeftEdgePosAcc[$iLVCols]
	For $i = 0 To $iLVCols - 1
		$aColumnWidths[$i] = _GUICtrlListView_GetColumnWidth( $idListView, $i )
		$aColLeftEdgePosAcc[$i] = $i ? $aColLeftEdgePosAcc[$i-1] + $aColumnWidths[$i] : 0
	Next

	; Edit control open and close events
	$idEditOpen = GUICtrlCreateDummy()
	$idEditClose = GUICtrlCreateDummy()

	; Handle horizontal page scrolls (ctrl + left/right)
	$idSetLeftMostVisibleCellFocused = GUICtrlCreateDummy()

	; Handle WM_NOTIFY messages for the ListView
	; Open Edit control on click/double click
	GUIRegisterMsg( $WM_NOTIFY, "WM_NOTIFY" )

	; Subclass callback functions
	Local $pListViewCallback = DllCallbackGetPtr( DllCallbackRegister( "ListViewCallback", "lresult", "hwnd;uint;wparam;lparam;uint_ptr;dword_ptr" ) )
	Local $pEditCallback = DllCallbackGetPtr( DllCallbackRegister( "EditCallback", "lresult", "hwnd;uint;wparam;lparam;uint_ptr;dword_ptr" ) )
	Local $pGuiCallback = DllCallbackGetPtr( DllCallbackRegister( "GuiCallback", "lresult", "hwnd;uint;wparam;lparam;uint_ptr;dword_ptr" ) )

	; Subclass ListView to handle messages related to Edit control
	_WinAPI_SetWindowSubclass( $hListView, $pListViewCallback, 9998, 0 )

	Local $idTest = GUICtrlCreateDummy()
	Local $aAccelKeys = [ [ "^b", $idTest ] ]
	GUISetAccelerators( $aAccelKeys )

	; Show GUI
	GUISetState( @SW_SHOW )

	; Message loop
	While 1
		Switch GUIGetMsg()
			Case $idTest
				MsgBox( 0, "Kill It", "Press Escape or Enter" )
			; Edit control open event from ListView
			Case $idEditOpen
				If $bEditOpen Then
					; If another Edit control is open then delete it
					_WinAPI_RemoveWindowSubclass( $hEdit, $pEditCallback, 9999 )
					_WinAPI_RemoveWindowSubclass( $hGui, $pGuiCallback, 9999 )
					_GUICtrlEdit_Destroy( $hEdit )
				EndIf
				; Create Edit control
				; Do not open control if the column is too narrow
				If $aColumnWidths[$iSubItem] < 20 Then ContinueLoop ; UDF => Error message
				; Make column visible if it's only partially visible along left/right edge of ListView
				Local $iCtrlWidth = MakeColumnVisible( $hListView, $iSubItem, $aRect, $iItem )
				$hEdit = _GUICtrlEdit_Create( $hListView, "", $aRect[0], $aRect[1], $iCtrlWidth, 20, $ES_AUTOHSCROLL )
				_GUICtrlEdit_SetText( $hEdit, _GUICtrlListView_GetItemText( $hListView, $iItem, $iSubItem ) )
				_GUICtrlEdit_SetSel( $hEdit, 0, -1 )
				; Create subclasses to handle Windows messages
				_WinAPI_SetWindowSubclass( $hEdit, $pEditCallback, 9999, 0 ) ; Handle messages from the Edit control
				_WinAPI_SetWindowSubclass( $hGui, $pGuiCallback, 9999, 0 )   ; Handle GUI messages related to Edit control
				; Set focus to Edit control                                  ; Subclasses are used only when Edit control is open
				_WinAPI_SetFocus( $hEdit )
				$bEditOpen = True

			; Edit control close event
			Case $idEditClose
				If Not $bEditOpen Then ContinueLoop
				If GUICtrlRead( $idEditClose ) Then _ ; Save value from Edit control in ListView item/subitem text
					_GUICtrlListView_SetItemText( $hListView, $iItem, _GUICtrlEdit_GetText( $hEdit ), $iSubItem )
				; Delete Edit control
				_WinAPI_RemoveWindowSubclass( $hEdit, $pEditCallback, 9999 )
				_WinAPI_RemoveWindowSubclass( $hGui, $pGuiCallback, 9999 )
				_GUICtrlEdit_Destroy( $hEdit )
				$bEditOpen = False

			; Handle horizontal page scrolls (ctrl + left/right)
			Case $idSetLeftMostVisibleCellFocused
				$iSubItemPrev = $iSubItem
				$iSubItem = _GUICtrlListView_SubItemHitTest( $hListView, 0, 0 )[1] + 1
				SetLeftCellFocused( $hListView, $iItem, $iSubItem )

			; Delete Edit ctrl if mouse is clicked in empty space of GUI
			; or if mouse is clicked for example on a button or a label.
			Case $GUI_EVENT_PRIMARYDOWN, $GUI_EVENT_SECONDARYDOWN
				If Not $bEditOpen Then ContinueLoop
				; Clicks in Edit control itself should not delete it
				Local $aPos = MouseGetWindowPos( $hListView )
				If Not ( $aPos[0] > $aRect[0] And $aPos[0] < $aRect[2] And $aPos[1] > $aRect[1] And $aPos[1] < $aRect[1] + 20 ) Then _
					GUICtrlSendToDummy( $idEditClose ) ; Delete Edit control

			Case $GUI_EVENT_CLOSE
				If Not $bEditEscape Then ExitLoop
				$bEditEscape = False
		EndSwitch
	WEnd

	; Cleanup
	_WinAPI_RemoveWindowSubclass( $hListView, $pListViewCallback, 9998 )
	GUIDelete()
EndFunc

; Open Edit control on click/double click
Func WM_NOTIFY( $hWnd, $iMsg, $wParam, $lParam )
	Local Static $bNotXP = Not ( @OSVersion = "WIN_XP" ), $tRect = DllStructCreate( $tagRECT ), $hBrush = _WinAPI_CreateSolidBrush( 0xFFFF00 ) ; Cyan
	Local $tNMHDR = DllStructCreate( $tagNMHDR, $lParam )
	Local $hWndFrom = HWnd( DllStructGetData( $tNMHDR, "hWndFrom" ) )
	Local $iCode = DllStructGetData( $tNMHDR, "Code" )
	Switch $hWndFrom
		Case $hListView
			Switch $iCode
				Case $NM_CUSTOMDRAW
					Local $tNMLVCustomDraw = DllStructCreate( $tagNMLVCUSTOMDRAW, $lParam )
					Local $dwDrawStage = DllStructGetData( $tNMLVCustomDraw, "dwDrawStage" )
					Switch $dwDrawStage                                ; Specifies the drawing stage
						; Stage 1
						Case $CDDS_PREPAINT                              ; Before the paint cycle begins
							Return $CDRF_NOTIFYITEMDRAW                    ; Stage 2 will be carried out

						; Stage 2
						Case $CDDS_ITEMPREPAINT                          ; Before an item is painted
							If Not _GUICtrlListView_GetItemState( $hListView, DllStructGetData( $tNMLVCUSTOMDRAW, "dwItemSpec" ), $LVIS_FOCUSED ) Then Return $CDRF_NEWFONT ; Default drawing of item
							$iItem = DllStructGetData( $tNMLVCUSTOMDRAW, "dwItemSpec" )
							Return $CDRF_NOTIFYSUBITEMDRAW                 ; Stage 3 will be carried out

						; Stage 3
						Case BitOR( $CDDS_ITEMPREPAINT, $CDDS_SUBITEM )  ; Before a subitem is painted
							Return $CDRF_NOTIFYPOSTPAINT                   ; Stage 4 will be carried out

						; Stage 4
						Case BitOR( $CDDS_ITEMPOSTPAINT, $CDDS_SUBITEM ) ; After a subitem has been painted
							If DllStructGetData( $tNMLVCustomDraw, "iSubItem" ) = $iSubItem Then
								Local $hDC = DllStructGetData( $tNMLVCUSTOMDRAW, "hdc" )                   ; Device context
								If $iSubItem Then
									$aRect = _GUICtrlListView_GetSubItemRect( $hListView, $iItem, $iSubItem ); Subitem rectangle
								Else ; $iSubItem = 0, the listview item, first column
									; This calculation of $aRect for the listview item is necessary to get
									; things to work properly if $LVS_EX_HEADERDRAGDROP style is applied.
									$aRect = _GUICtrlListView_GetItemRect( $hListView, $iItem, 2 )           ; 2 - The bounding rectangle of the item text
									$aRect[0] -= 4
								EndIf
								If $bNotXP And $iSubItem = 0 And $aColumnOrder[1] <> 0 Then
									DllStructSetData( $tRect, "Left", $aRect[0] )                            ; Margin for first column if dragged to another position
								Else
									DllStructSetData( $tRect, "Left", $aRect[0]+4 )
								EndIf
								DllStructSetData( $tRect, "Top",    $aRect[1] )
								DllStructSetData( $tRect, "Right",  $aRect[2] )
								DllStructSetData( $tRect, "Bottom", $aRect[3] )
								_WinAPI_FillRect( $hDC, $tRect, $hBrush )                                  ; Fill subitem background
								_WinAPI_SetBkMode( $hDC, $TRANSPARENT )                                    ; Transparent background
								_WinAPI_SetTextColor( $hDC, 0x000000 )                                     ; Set black text color
								DllStructSetData( $tRect, "Left", DllStructGetData( $tRect, "Left" ) + 2 ) ; Adjust rectangle
								If $bNotXP Then DllStructSetData( $tRect, "Top", DllStructGetData( $tRect, "Top" ) + 2 )
								_WinAPI_DrawText( $hDC, _GUICtrlListView_GetItemText( $hListView, $iItem, $iSubItem ), $tRect, $DT_WORD_ELLIPSIS ) ; Draw text
							EndIf
							Return $CDRF_NEWFONT                           ; $CDRF_NEWFONT must be returned after changing font or colors
					EndSwitch
				Case $NM_CLICK
					If Not $bDoNotOpenControl And Not $bEditOpenOnDoubleClick And $iItem > -1 And $iSubItem > -1 Then _
						GUICtrlSendToDummy( $idEditOpen ) ; Send message to open Edit control
				Case $NM_DBLCLK
					If $bEditOpenOnDoubleClick And $iItem > -1 And $iSubItem > -1 Then _
						GUICtrlSendToDummy( $idEditOpen ) ; Send message to open Edit control
				Case $NM_SETFOCUS
					$bLVHasFocus = True
				Case $NM_KILLFOCUS
					$bLVHasFocus = False
					$bEditOpen = False
					$bDoNotOpenControl = False
					$bEditEscape = False
					$bControl = False
			EndSwitch

		Case $hHeader
			Switch $iCode
				Case $HDN_ENDDRAG
					$aColumnOrder = 0
				Case $HDN_ENDTRACKW
					$aColLeftEdgePosAcc = 0
					Local $tNMHEADER = DllStructCreate( $tagNMHEADER, $lParam ), $tHDITEM = DllStructCreate( $tagHDITEM, DllStructGetData( $tNMHEADER, "pItem" ) )
					If BitAND( DllStructGetData( $tHDITEM, "Mask" ), $HDI_WIDTH ) Then $aColumnWidths[DllStructGetData( $tNMHEADER, "Item" )] = DllStructGetData( $tHDITEM, "XY" )
				Case $NM_RELEASEDCAPTURE
					If Not IsArray( $aColumnOrder ) Or Not IsArray( $aColLeftEdgePosAcc ) Then _
						CalcColumnOrderArrays( $hListView )
			EndSwitch
	EndSwitch
	Return $GUI_RUNDEFMSG
	#forceref $hWnd, $iMsg, $wParam
EndFunc

; Handle ListView messages related to Edit control
Func ListViewCallback( $hWnd, $iMsg, $wParam, $lParam, $iSubclassId, $pData )
	;Static $bControl = False
	Switch $iMsg
		; Open Edit control on Enter key
		Case $WM_GETDLGCODE
			Switch $wParam
				Case $VK_RETURN
					If Not $bDoNotOpenControl And $bEditOpenOnEnter And $iItem > -1 And $iSubItem > -1 Then
						_GUICtrlListView_SetItemSelected( $hListView, -1, False )
						_GUICtrlListView_SetItemSelected( $hListView, $iItem )
						GUICtrlSendToDummy( $idEditOpen )
					EndIf
			EndSwitch

		; Manage multiple selections
		; Prevent Edit control from opening
		; Manage keyboard events
		Case $WM_KEYDOWN
			Switch $wParam
				Case $VK_SHIFT, $VK_CONTROL
					$bControl = $wParam = $VK_CONTROL ? True : False
					$bDoNotOpenControl = True
				; Manage keyboard events
				Case $VK_LEFT, $VK_RIGHT
					If $iItem > -1 And $bLVHasFocus And ( Not $bDoNotOpenControl Or $bControl ) Then
						If $bControl Then
							; Suppress repeated right page scrolls on the rightmost page
							If $iSubItem = $iSubItemPrev And $wParam = $VK_RIGHT Then Return 0
							; Let the listview perform the default code (scroll a page left/right)
							; Then set the focused cell to the leftmost visible cell
							GUICtrlSendToDummy( $idSetLeftMostVisibleCellFocused )
						Else
							Local $i = $wParam = $VK_LEFT ? SetLeftCellFocused( $hListView, $iItem, $iSubItem ) _
							                              : SetRightCellFocused( $hListView, $iItem, $iSubItem )
							Return 0 ; Skip listview default code (scroll 5 pixels left/right)
						EndIf
					EndIf
				Case $VK_SPACE
					If Not $bDoNotOpenControl And $bEditOpenOnSpace And $iItem > -1 And $iSubItem > -1 Then
						_GUICtrlListView_SetItemSelected( $hListView, -1, False )
						GUICtrlSendToDummy( $idEditOpen )
					EndIf
			EndSwitch
		Case $WM_KEYUP
			Switch $wParam
				Case $VK_SHIFT, $VK_CONTROL
					$bDoNotOpenControl = False
					$bControl = False
			EndSwitch

		; Left click in ListView
		; Sent on single and double click
		; Determines item/subitem of the cell that's clicked
		Case $WM_LBUTTONDOWN
			If $bEditOpen Then
				; If another Edit control is open then delete it
				_GUICtrlEdit_Destroy( $hEdit )
				GUICtrlSendToDummy( $idEditClose )
			EndIf
			Local $aHit = _GUICtrlListView_SubItemHitTest( $hListView )
			If $aHit[0] > -1 And $aHit[1] > -1 Then
				$iItem = $aHit[0]
				$iSubItem = $aHit[1]
				_GUICtrlListView_RedrawItems( $hListView, $iItem, $iItem )
			EndIf

		; Delete the Edit control on right click in ListView and on
		; left or right click in non-client ListView area (Scrollbars).
		Case $WM_RBUTTONDOWN, $WM_NCLBUTTONDOWN, $WM_NCRBUTTONDOWN
			_GUICtrlEdit_Destroy( $hEdit )
			GUICtrlSendToDummy( $idEditClose )
	EndSwitch

	; Call next function in subclass chain
	Return DllCall( "comctl32.dll", "lresult", "DefSubclassProc", "hwnd", $hWnd, "uint", $iMsg, "wparam", $wParam, "lparam", $lParam )[0]
	#forceref $iSubclassId, $pData
EndFunc

; Handle messages from the Edit control
Func EditCallback( $hWnd, $iMsg, $wParam, $lParam, $iSubclassId, $pData )
	Switch $iMsg
		; Dialog codes
		Case $WM_GETDLGCODE
			Switch $wParam
				Case $VK_TAB    ; Close
					GUICtrlSendToDummy( $idEditClose )
				Case $VK_RETURN ; Accept and close
					GUICtrlSendToDummy( $idEditClose, True )
				Case $VK_ESCAPE ; Close
					GUICtrlSendToDummy( $idEditClose )
					$bEditEscape = True
			EndSwitch

		; Double click in Edit control
		Case $WM_LBUTTONDBLCLK
			GUICtrlSendToDummy( $idEditClose, True )
	EndSwitch

	; Call next function in subclass chain
	Return DllCall( "comctl32.dll", "lresult", "DefSubclassProc", "hwnd", $hWnd, "uint", $iMsg, "wparam", $wParam, "lparam", $lParam )[0]
	#forceref $iSubclassId, $pData
EndFunc

; Handle GUI messages related to Edit control
Func GuiCallback( $hWnd, $iMsg, $wParam, $lParam, $iSubclassId, $pData )
	Switch $iMsg
		; Delete Edit control on left or right mouse click in non-client GUI area and on GUI deactivate
		Case $WM_NCLBUTTONDOWN, $WM_NCRBUTTONDOWN, $WM_ACTIVATE
			_GUICtrlEdit_Destroy( $hEdit )
			GUICtrlSendToDummy( $idEditClose )
	EndSwitch

	; Call next function in subclass chain
	Return DllCall( "comctl32.dll", "lresult", "DefSubclassProc", "hwnd", $hWnd, "uint", $iMsg, "wparam", $wParam, "lparam", $lParam )[0]
	#forceref $iSubclassId, $pData
EndFunc
