#include-once
#include <GuiScrollBars.au3>

Global $aColumnOrder = 0, $aColumnOrderReverse, $aColumnWidths
Global  $aColLeftEdgePosAcc ; Accumulated left edge positions of columns in left to right column order

Func MakeColumnVisible( $hListView, $iSubItem, ByRef $aRect, $iItem = -1 )
	Local $iLvWidth = WinGetClientSize( $hListView )[0]
	Local $aRct = _GUICtrlListView_GetSubItemRect( $hListView, $iItem, $iSubItem )
	If $aRct[0] < 0 Then
		; Make entire column visible in left part of ListView
		; If left part of the column isn't visible align to left edge
		_GUICtrlListView_Scroll( $hListView, $aRct[0], 0 )
	Else
		; Make entire column visible in right part of ListView
		If $aRct[2] - $aRct[0] >= $iLvWidth Then
			; If the column is wider than the listview align to left edge
			_GUICtrlListView_Scroll($hListView, $aRct[0], 0)
		ElseIf $aRct[2] > $iLvWidth Then
			; If right part of the column isn't visible align to right edge
			_GUICtrlListView_Scroll( $hListView, $aRct[2] - $iLvWidth, 0 )
		EndIf
	EndIf
	; Redraw ListView item
	_GUICtrlListView_EnsureVisible( $hListView, $iItem )
	_GUICtrlListView_RedrawItems( $hListView, $iItem, $iItem )

	; Necessary to recalculate $aRect in examples without keyboard support
	; In examples with keyboard support this is done in the custom draw code
	If $iItem <> -1 Then 
		If $iSubItem Then
			$aRect = _GUICtrlListView_GetSubItemRect( $hListView, $iItem, $iSubItem )
			$aRect[2] = $aRect[0] + $aColumnWidths[$iSubItem]
		Else
			$aRect = _GUICtrlListView_GetItemRect( $hListView, $iItem, 2 ) ; 2 - The bounding rectangle of the item text
			$aRect[0] -= 4
		EndIf
	EndIf

	Return $aRect[2] - $aRect[0] < $iLvWidth ? $aRect[2] - $aRect[0] : $iLvWidth
EndFunc

Func CalcColumnOrderArrays( $hListView )
	If Not IsArray( $aColumnOrder ) Then
		; Calculate $aColumnOrderReverse array
		$aColumnOrder = _GUICtrlListView_GetColumnOrderArray( $hListView ) ; $aColumnOrder specifies $iSubItem with column number as index
		$aColumnOrderReverse = $aColumnOrder                               ; $aColumnOrderReverse specifies column number with $iSubItem as index
		For $i = 1 To $aColumnOrder[0]
			$aColumnOrderReverse[$aColumnOrder[$i]] = $i
		Next
		; Calculate accumulated left edge positions of columns in left to right column order
		$aColLeftEdgePosAcc = $aColumnOrder
		$aColLeftEdgePosAcc[0] = 0
		For $i = 1 To $aColumnOrder[0] - 1
			$aColLeftEdgePosAcc[$i] = $aColLeftEdgePosAcc[$i-1] + $aColumnWidths[$aColumnOrder[$i]]
		Next
	ElseIf Not IsArray( $aColLeftEdgePosAcc ) Then
		; Calculate accumulated left edge positions of columns in left to right column order
		$aColLeftEdgePosAcc = $aColumnOrder
		$aColLeftEdgePosAcc[0] = 0
		For $i = 1 To $aColumnOrder[0] - 1
			$aColLeftEdgePosAcc[$i] = $aColLeftEdgePosAcc[$i-1] + $aColumnWidths[$aColumnOrder[$i]]
		Next
	EndIf
EndFunc

Func SetLeftCellFocused( $hListView, $iItem, ByRef $iSubItem )
	If $aColumnOrderReverse[$iSubItem] > 1 Then
		Local $i = $aColumnOrder[$aColumnOrderReverse[$iSubItem]-1]
		While $aColumnOrderReverse[$i] > 1 And Not $aColumnWidths[$i]
			$i = $aColumnOrder[$aColumnOrderReverse[$i]-1]
		WEnd
		If $aColumnWidths[$i] Then $iSubItem = $i
	EndIf

	; Make entire column visible in left part of ListView
	Local $iLvWidth = WinGetClientSize( $hListView )[0]
	Local $aRect = _GUICtrlListView_GetSubItemRect( $hListView, $iItem, $iSubItem )
	If $aRect[0] > $iLvWidth Then
		; Marked cell/column on right side of visible columns
		; (Horzontal scroll button has been dragged to the left)
		If $aRect[2] - $aRect[0] >= $iLvWidth Then
			; If the column is wider than the listview align to left edge
			_GUICtrlListView_Scroll( $hListView, $aRect[0], 0 )
		Else
		; Align the cell/column to right edge of the listview
		_GUICtrlListView_Scroll( $hListView, $aRect[2] - $iLvWidth, 0 )
		EndIf
	ElseIf $aRect[0] < 0 Then
		; If left part of the column isn't visible align to left edge
		_GUICtrlListView_Scroll( $hListView, $aRect[0], 0 )
	EndIf
	; Redraw ListView item
	_GUICtrlListView_EnsureVisible( $hListView, $iItem )
	_GUICtrlListView_RedrawItems( $hListView, $iItem, $iItem )
EndFunc

Func SetRightCellFocused( $hListView, $iItem, ByRef $iSubItem )
	If $aColumnOrderReverse[$iSubItem] < $aColumnOrder[0] Then
		Local $i = $aColumnOrder[$aColumnOrderReverse[$iSubItem]+1]
		While $aColumnOrderReverse[$i] < $aColumnOrder[0] And Not $aColumnWidths[$i]
			$i = $aColumnOrder[$aColumnOrderReverse[$i]+1]
		WEnd
		If $aColumnWidths[$i] Then $iSubItem = $i
	EndIf

	; Make entire column visible in right part of ListView
	Local $iLvWidth = WinGetClientSize( $hListView )[0]
	Local $aRect = _GUICtrlListView_GetSubItemRect( $hListView, $iItem, $iSubItem )
	If $aRect[2] < 0 Then
		; Marked cell/column on left side of visible columns
		; (Horzontal scroll button has been dragged to the right)
		; In this case align the cell/column to left edge of the listview
		_GUICtrlListView_Scroll( $hListView, $aRect[0], 0 )
	ElseIf $aRect[2] - $aRect[0] >= $iLvWidth Then
		; If the column is wider than the listview align to left edge
		_GUICtrlListView_Scroll( $hListView, $aRect[0], 0 )
	ElseIf $aRect[2] > $iLvWidth Then
		; If right part of the column isn't visible align to right edge
		_GUICtrlListView_Scroll( $hListView, $aRect[2] - $iLvWidth, 0 )
	EndIf
	; Redraw ListView item
	_GUICtrlListView_EnsureVisible( $hListView, $iItem )
	_GUICtrlListView_RedrawItems( $hListView, $iItem, $iItem )
EndFunc

; Get mouse pos relative to window
Func MouseGetWindowPos( $hWindow )
	Local $aPos = MouseGetPos()
	Local $tPoint = DllStructCreate( "int X;int Y" )
	DllStructSetData( $tPoint, "X", $aPos[0] )
	DllStructSetData( $tPoint, "Y", $aPos[1] )
	_WinAPI_ScreenToClient( $hWindow, $tPoint )
	$aPos[0] = DllStructGetData( $tPoint, "X" )
	$aPos[1] = DllStructGetData( $tPoint, "Y" )
	Return $aPos
EndFunc
